SystemOrganization addCategory: #GODCall!

GODData subclass: #GCConference
	instanceVariableNames: 'identifier name acronym edition categories keywords url importantDates venue rating'
	classVariableNames: 'SpreadsheetMap'
	poolDictionaries: ''
	category: 'GODCall'!

!GCConference commentStamp: 'cms 11/21/2014 19:27' prior: 0!
A GCConference represents a conference

Instance Variables
	acronym:		<Object>
	categories:		<Object>
	edition:		<Object>
	identifier:		<Object>
	importantDates:		<Object>
	keywords:		<Object>
	name:		<Object>
	rating:		<Object>
	url:		<Object>
	venue:		<Object>
!

!GCConference class methodsFor: 'spreadsheet' stamp: 'fabcm 11/29/2014 19:49'!
asSpreadsheet: anCollectionOfConferences
	"given a collection of conferences it returns a spreadsheet in which each line is a conference"
	|spreadsheet sheet rowIndex|
	spreadsheet := SSSpreadsheetData new.
	sheet := spreadsheet createSheetWithName: 'conferences'.

	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'acronym') withValue: 'Acronym'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'name') withValue: 'Name'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'edition') withValue: 'Edition'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at:'deadlineDate') withValue: 'Deadline'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'startDate') withValue: 'Start'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'endDate' ) withValue: 'End'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'qualis') withValue: 'Qualis'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'hIndex') withValue: 'h-index'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'country') withValue: 'Country'.
	sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'city') withValue: 'City'.
	"sheet createCellAtRow: 1 atColumn: (GCConference dictionary at: 'place') withValue: 'Place'."


	1 to: (anCollectionOfConferences size) do: [:index |
		rowIndex := index + 1.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'acronym') withValue: (GCConference encode: (anCollectionOfConferences at: index) acronym).
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'name') withValue: (GCConference encode: (anCollectionOfConferences at: index) name).
		"categoriesAsString := GCConference joinCollection: (anCollectionOfConferences at: index) categories.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'categories') withValue: categoriesAsString."
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'edition') withValue: (GCConference encode:((anCollectionOfConferences at: index) edition)).
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at:'deadlineDate') withValue: (anCollectionOfConferences at: index) importantDates deadlineDate.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'startDate') withValue: (anCollectionOfConferences at: index) importantDates startDate.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'endDate' ) withValue: (anCollectionOfConferences at: index) importantDates endDate.
		"keywordsAsString := GCConference joinCollection: (anCollectionOfConferences at: index) keywords.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'keywords') withValue: keywordsAsString.
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'url') withValue: (anCollectionOfConferences at: index) url."
		(anCollectionOfConferences at: index) rating ifNotNil:[ :rating|
			sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'qualis') withValue: rating qualis.
			sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'hIndex') withValue: rating hIndex.
		] ifNil:[
			sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'qualis') withValue: nil.
			sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'hIndex') withValue: nil.
		].
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'city') withValue: (GCConference encode:((anCollectionOfConferences at: index) venue city)).
		sheet createCellAtRow: rowIndex atColumn: (GCConference dictionary at: 'country') withValue: (GCConference encode:((anCollectionOfConferences at: index) venue country)). 
	].
	
	^ spreadsheet.	! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'cms 11/29/2014 17:19'!
dictionary
	"returns a mapping of GCConference attributes to spreadsheet columns identifiers"
		self makeDictionary.
		
	^ SpreadsheetMap.! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'fabcm 11/29/2014 19:46'!
encode: aString
"TODO comentrios"

	| string |

	aString ifNotNil:[ :aux |
		string := aux copyReplaceAll: '&' with: '&amp;'.
	] ifNil:[
		string := nil.
	].



	^ string! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'cms 11/21/2014 19:40'!
from: spreadsheet	
	"creates a list of conferences from a spreadsheet (each line must contain a conference that the column mapping respect the one in our dictionary"
	|sheet conferencesCollection conference venue importantDates spreadsheetMap categories keywords| 
	conferencesCollection := OrderedCollection new.
	spreadsheetMap := GCConference dictionary.
		sheet := spreadsheet getSheet: 1. 
		sheet do: [:row |
 			conference := GCConference new.
			conference acronym: (row getCell: (spreadsheetMap at: 'acronym')) value.
			conference name: (row getCell: (spreadsheetMap at: 'name')) value.
			categories := GCConference parse: (row getCell: (spreadsheetMap at: 'categories')) value.
			conference categories: categories.
			keywords := GCConference parse: (row getCell: (spreadsheetMap at: 'keywords')) value.
			conference keywords: keywords.
			conference rating: (row getCell: (spreadsheetMap at: 'rating')) value.
			conference edition: (row getCell: (spreadsheetMap at: 'edition')) value.
			conference url: (row getCell: (spreadsheetMap at: 'url')) value.
			importantDates := GCImportantDates new.
			importantDates deadlineDate: (row getCell: (spreadsheetMap at: 'deadlineDate')) value.
			importantDates startDate: (row getCell: (spreadsheetMap at: 'startDate')) value.
			importantDates endDate: (row getCell: (spreadsheetMap at: 'endDate')) value.
			venue := GCVenue new.
			venue city: (row getCell: (spreadsheetMap at: 'city')) value.
			venue country: (row getCell: (spreadsheetMap at: 'country')) value.
			venue place: (row getCell: (spreadsheetMap at: 'place')) value.
			conference venue: venue.
			conference importantDates: importantDates.	
			conferencesCollection add: conference.     
		].
	^ conferencesCollection.
      
	
	
! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'cms 11/3/2014 23:19'!
joinCollection: anCollection
	"Creates a string with the items on anCollection separated by separator"
	(anCollection = nil) ifTrue:[
		^ String new.
	].
	^ String streamContents: [:stream | anCollection asStringOn: stream delimiter: '&']
	! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'fabcm 11/29/2014 10:44'!
makeDictionary
	"makes a mapping of GCConference attributes to spreadsheet columns identifiers"

	SpreadsheetMap := Dictionary new.
	SpreadsheetMap at: 'identifier' put: 'A'.
	SpreadsheetMap at: 'acronym' put: 'B'.
	SpreadsheetMap at: 'name' put: 'C'.
	SpreadsheetMap at: 'edition' put: 'D'.
	SpreadsheetMap at: 'categories' put: 'E'.
	SpreadsheetMap at: 'deadlineDate' put: 'F'.
	SpreadsheetMap at: 'startDate' put: 'G'.
	SpreadsheetMap at: 'endDate' put: 'H'.
	SpreadsheetMap at: 'url' put: 'I'.
	SpreadsheetMap at: 'country' put: 'J'.
	SpreadsheetMap at: 'city' put: 'K'.
	SpreadsheetMap at: 'place' put: 'L'.
	SpreadsheetMap at: 'keywords' put: 'M'.
	SpreadsheetMap at: 'qualis' put: 'N'.
	SpreadsheetMap at: 'hIndex' put: 'O'.
! !

!GCConference class methodsFor: 'normalizing' stamp: 'pfsm 11/26/2014 15:15'!
normalizeAcronym: anAcronym
	"put anAcronym into normalized form"

	|badWords normalizedAcronym|
	
"	badWords:= #('ieee' 'acm' 'issue' 'volume' 'special' 'tutorial' 'panel' '(' ')')."
	badWords:= #('issue' 'volume' 'special' 'tutorial' 'panel').

	normalizedAcronym := anAcronym asLowercase.

	badWords do:[:bad | normalizedAcronym := normalizedAcronym copyReplaceAll: bad with: String empty].
	

	^normalizedAcronym onlyLetters.
	"^normalizedAcronym withBlanksTrimmed."

! !

!GCConference class methodsFor: 'spreadsheet' stamp: 'cms 11/3/2014 23:14'!
parse: anCollectionAsString 
	"undo process made by joinCollection: anCollection with: separator"
	^ anCollectionAsString findTokens: '&'.! !

!GCConference methodsFor: 'comparing' stamp: 'pfsm 11/24/2014 19:19'!
= aConference
	"compare two conferences"
	
	(self == aConference)	 ifTrue: [^ true].	"stop recursion"
"	(self identifier = aConference identifier) ifFalse: [^ false]."
	(self acronym = aConference acronym) ifFalse: [^ false].
	(self name = aConference name) ifFalse: [^ false].
	(self edition = aConference edition) ifFalse: [^ false].
	(self categories = aConference categories) ifFalse: [^ false].
	((self url) asString  = (aConference url) asString) ifFalse: [^ false].
	(self importantDates = aConference importantDates) ifFalse: [^ false].
	(self venue = aConference venue) ifFalse: [^ false].

^ true.

! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
acronym
	^ acronym! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 11/29/2014 22:13'!
acronym: conferenceAcronym
	acronym := conferenceAcronym.

	"create an identifier using the conference's acronym"
	identifier := GCConference normalizeAcronym: conferenceAcronym.
	self addKeywords: (Set with: acronym asLowercase).! !

!GCConference methodsFor: 'adding' stamp: 'pfsm 11/6/2014 21:42'!
addCategories: aSetOfCategories
	"add aSetOfCategories to the set of current categories of the conference"
	
	(categories)
		ifNil: [categories := aSetOfCategories]
		ifNotNil: [categories:= (categories union: aSetOfCategories)].! !

!GCConference methodsFor: 'adding' stamp: 'pfsm 11/3/2014 21:37'!
addKeywords: aSetOfKeywords
	"add aSetOfKeywords to the set of current keywords of the conference"
	
	(keywords = nil)
		ifTrue: [keywords := aSetOfKeywords]
		ifFalse: [keywords:= (keywords union: aSetOfKeywords)].! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
categories
	^ categories! !

!GCConference methodsFor: 'accessing' stamp: 'fabcm 11/29/2014 15:05'!
categories: conferenceCategories
	categories := conferenceCategories.
	self addKeywords: conferenceCategories.! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 22:10'!
edition

	^ edition! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 22:11'!
edition: anEdition
	"anEdition is usually the year"
	edition := anEdition! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:12'!
identifier

	^ identifier! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
importantDates
	 ^ importantDates! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
importantDates: conferenceImportantDates
	importantDates := conferenceImportantDates! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:12'!
keywords

	^ keywords! !

!GCConference methodsFor: 'accessing' stamp: 'fabcm 11/29/2014 12:57'!
keywords: conferenceKeywords

	tags := conferenceKeywords asOrderedCollection.
	self addKeywords: conferenceKeywords! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
name
	^ name! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 11/6/2014 19:17'!
name: conferenceName
	name := conferenceName.

	"create a list of keywords using the conference's name"
	self addKeywords: (GODCall extractKeywordsFrom: conferenceName). 
		
	! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:12'!
rating

	^ rating! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:15'!
rating: aRating

	rating := aRating! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 16:07'!
url
	^ url! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 16:08'!
url: aUrl

	url:= aUrl! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
venue
	 ^ venue! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
venue: conferenceVenue
	venue := conferenceVenue! !

WAComponent subclass: #GCFirstPage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCFirstPage commentStamp: 'fabcm 11/29/2014 18:23' prior: 0!
A GCFirstPage is a component responsible for rendering the first page with seaside.

This page has a formulary that receives the user's search preferences.!

!GCFirstPage class methodsFor: 'class initialization' stamp: 'fabcm 11/28/2014 14:49'!
initialize
	WAAdmin register: self asApplicationAt: 'godcall'! !

!GCFirstPage methodsFor: 'as yet unclassified' stamp: 'fabcm 11/28/2014 15:16'!
renderContentOn: html

	|page|
	page := GODCall new initialize: self; renderFirstPage.	
	page render: html.
	! !

WAComponent subclass: #GCSecondPage
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSecondPage commentStamp: 'fabcm 11/29/2014 18:22' prior: 0!
A GCSecondPage is a component responsible for rendering the second page with seaside.

This page shows a table with conferences found.!

!GCSecondPage methodsFor: 'as yet unclassified' stamp: 'fabcm 11/28/2014 17:55'!
object: anObject

	object:= anObject.! !

!GCSecondPage methodsFor: 'as yet unclassified' stamp: 'fabcm 11/28/2014 20:15'!
renderContentOn: html

	| page |
	object component: self.
	page := object renderSecondPage.
	page render: html.! !

WAComponent subclass: #GCThirdPage
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCThirdPage commentStamp: 'fabcm 11/29/2014 18:24' prior: 0!
A GCThirdPage is a component responsible for rendering the third page with seaside.

This page shows more details of a conference.!

!GCThirdPage methodsFor: 'trash' stamp: 'fabcm 11/28/2014 19:41'!
conference: aConference
! !

!GCThirdPage methodsFor: 'as yet unclassified' stamp: 'fabcm 11/28/2014 19:41'!
object: aObject

	object := aObject.! !

!GCThirdPage methodsFor: 'as yet unclassified' stamp: 'fabcm 11/28/2014 19:40'!
renderContentOn: html

	| page |
	page := object renderThirdPage.
	page render: html.! !

Object subclass: #GCConferenceResponse
	instanceVariableNames: 'conference component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCConferenceResponse commentStamp: 'fabcm 11/29/2014 18:25' prior: 0!
A GCConferenceResponse is a object responsible for rendering the conference details in the third page.!

!GCConferenceResponse methodsFor: 'trash' stamp: 'fabcm 11/28/2014 19:29'!
conference: aConference
! !

!GCConferenceResponse methodsFor: 'initialize' stamp: 'fabcm 11/29/2014 18:28'!
initialize: aComponent withConference: aConference
" The component is necessary to comunicate with seaside and the conference is the one which details will be show."
	
	component := aComponent.
	conference := aConference.

! !

!GCConferenceResponse methodsFor: 'trash' stamp: 'fabcm 11/28/2014 19:28'!
object: anObject
! !

!GCConferenceResponse methodsFor: 'interface' stamp: 'fabcm 11/29/2014 20:13'!
renderThirdPage
"This message constructs the WEBPage with the details of conference"


	| page iForm iname contents iedition icategories collection ikeywords iurl idead istart iend icity icountry iqualis ihindex icontent|
	
	iname := WEBParagraph new.
	iname value: 'Name: (', conference acronym, ') ', conference name.
	
	iedition := WEBParagraph new.
	iedition value: 'Edition: ', conference edition..
	
	contents := String empty.
	collection := conference categories asOrderedCollection.
	[1 to: (collection size - 1) do:[ :index | contents := contents, (collection at: index), ', ' ].] on: MessageNotUnderstood do: [].
	[contents := contents, (collection at: (collection size)), '.'] on: MessageNotUnderstood do: [].
	icategories := WEBParagraph new.
	icategories value: 'Categories: ', contents.
	
	contents := String empty.
	collection := conference tags.
	collection ifNotEmpty: [ 
		1 to: (collection size - 1) do:[ :index | contents := contents, (collection at: index), ', ' ].
		contents := contents, (collection at: (collection size)), '.'.
	].
	ikeywords := WEBParagraph new.
	ikeywords value: 'Keywords: ', contents.
	
	iurl := WEBAnchor new.
	iurl label: conference url.
	iurl value: conference url.
	
	idead := WEBParagraph new.
	idead value: 'Deadline: ', (conference importantDates deadlineDate).
	
	istart := WEBParagraph new.
	istart value: 'Start: ', (conference importantDates startDate).
	
	iend := WEBParagraph new.
	iend value: 'End: ', (conference importantDates endDate).
	
	icountry := WEBParagraph new.
	icountry value: 'Country: ', (conference venue country).
	
	icity := WEBParagraph new.
	icity value: 'City: ', (conference venue city).
	
	
	iqualis := WEBParagraph new.
	ihindex := WEBParagraph new.
	
	conference rating ifNotNilDo:[ : rating |
		iqualis value: 'Qualis: ', ( rating qualis).
		ihindex value: 'h-index: ', ( rating hIndex).
	] ifNil:[
		iqualis value: 'Qualis: '.
		ihindex value: 'h-index: '.
	].

	icontent := WEBParagraph new.
	icontent value: conference content.

	iForm := WEBForm new.
	iForm initialize;
		add: iname;
		add: icategories;
		add: ikeywords;
		add: iurl;
		add: idead;
		add: istart;
		add: iend;
		add: icountry;
		add: icity;
		add: iqualis;
		add: ihindex.
	"	add: icontent."  " THIS LINE IS COMMENTD BECAUSE THE CONTENT ISNT BEING WELL RENDERED"
	
	page:= WEBPage new.
	page initialize.
	page title: 'Conference Details'.
	page add: iForm.
	
	^ page.! !

!GCConferenceResponse methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:29'!
save
"This message is triggered when the uses clicks in the button on second page and is responsible to call the third page."

	component call: (GCThirdPage new object: self).! !

Object subclass: #GCCrawler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCCrawler class methodsFor: 'accessing' stamp: 'fabcm 11/28/2014 16:23'!
categories
	"returns all categories used by source crawlers"
	|sources categories|
	sources := GCSourceCreator createAll.
	categories := SortedCollection new.
	sources do: [ :source |
		source categories values size ~= 0 ifTrue:[
			categories addAll: source categories values.
		].
	].
	^ categories.! !

!GCCrawler class methodsFor: 'merging' stamp: 'pfsm 11/26/2014 13:04'!
combine: conference1 with: conference2
	"add to conference1 the keywords, categories and content of conference2"
	
	(conference2 edition = conference1 edition)
		ifTrue:[
					((conference2 keywords) notNil ) ifTrue:[conference1 addKeywords: (conference2 keywords)].
					((conference2 categories) notNil) ifTrue:[conference1 addCategories: (conference2 categories)].
					((conference2 content) notNil) ifTrue:[conference1 content: (conference2 content)].
				].
	

	(conference2 origin) 
		ifNotNil:[conference1 origin: (conference1 origin,'+',conference2 origin)].

	(conference2 rating)
		ifNotNil:[conference1 rating: (conference2 rating)].

^ conference1.! !

!GCCrawler class methodsFor: 'merging' stamp: 'cms 11/9/2014 12:58'!
createDictionary: anConferenceColletion
	"transforms a conference collection in dictionary using identifier as a key"
	|dictionary|
	dictionary := Dictionary new.
	
		anConferenceColletion do: [ :conference |
			dictionary at: (conference identifier) put: conference.
		].
	^ dictionary.
! !

!GCCrawler class methodsFor: 'merging' stamp: 'pfsm 11/26/2014 01:11'!
merge: conferences1 with: conferences2
	"for each conference associated to an object obj1 in conference1 and to an object obj2 in conference2, construct a new conference object that combines the information in obj1 and obj2"

	| mergedConferences resultConferences conf2 dictConferences2|
	mergedConferences := OrderedCollection new.
	resultConferences := OrderedCollection new.
	dictConferences2 := self createDictionary: conferences2.

	conferences1 do: [ :conf | 
		(dictConferences2 includesKey: (conf identifier))
		ifTrue: [
			conf2 := dictConferences2  at: conf identifier.
			dictConferences2 removeKey: (conf identifier).
			self combine: conf with: conf2.
		].
		mergedConferences add: conf.
	].
		
"	mergedConferences addAll: (dictConferences2 values)."
	(dictConferences2 values) do:[:conf | (conf origin ~= 'qualis') ifTrue:[ mergedConferences add: conf ]].
	^ mergedConferences.
! !

!GCCrawler class methodsFor: 'initialize' stamp: 'pfsm 11/26/2014 15:26'!
run
	" search for conferences in web, merge information with quallis, and save in database"
	| sourcesCollection godConferenceCollection|

	sourcesCollection := GCSourceCreator createAll.
      godConferenceCollection := OrderedCollection new.
     "	sourcesCollection do: [ :source |
				godConferenceCollection := self merge: godConferenceCollection with: (source crawler crawl: (source categories)).
		]."
	sourcesCollection 
		do: [ :source | godConferenceCollection := self merge: godConferenceCollection with:  (self selectLatest: (source crawler crawl: (source categories)))].
	
	GDBClientController initializeSession.
	GDBClientController resetConferenceData.
	GDBClientController saveConferences: godConferenceCollection.
	GDBClientController release.! !

!GCCrawler class methodsFor: 'merging' stamp: 'pfsm 11/29/2014 12:48'!
selectLatest: confCollection 
	"select the latest edition of conferences with the same identifier in confCollection"

	| dictionary selectedConferences latestDate latestConf latestConferences |
	latestConferences := OrderedCollection new.
	dictionary := self createDictionary: confCollection.
	dictionary keys
		do: [:key | 
						selectedConferences := confCollection select: [:conf | (conf identifier = key) ].
						latestDate := '01 January 1901' asDate.
						selectedConferences
							do: [:conf |	(conf importantDates)
												ifNil:[	latestConf := conf]
												ifNotNil:[	((conf importantDates) startDate)
																	ifNil:[latestConf := conf]
																	ifNotNil:[(conf importantDates) startDate > latestDate
																					ifTrue: [	latestConf := conf.
																								latestDate := (conf importantDates) startDate
																							   ]
																				]
															]
								].
						latestConferences add: latestConf].
	^ latestConferences! !

Object subclass: #GCImportantDates
	instanceVariableNames: 'startDate endDate deadlineDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCImportantDates commentStamp: 'cms 11/21/2014 19:27' prior: 0!
A GCImportantDates holds deadline date, end date and start date of a conference.

Instance Variables
	deadlineDate:		<Object>
	endDate:		<Object>
	startDate:		<Object>
!

!GCImportantDates methodsFor: 'comparing' stamp: 'pfsm 10/16/2014 16:30'!
= anImportantDates
	"compare two importantDates"
	
	self == anImportantDates ifTrue: [^ true].	"stop recursion"
	(self startDate = anImportantDates startDate) ifFalse: [^ false].
	(self endDate = anImportantDates endDate) ifFalse: [^ false].
	(self deadlineDate = anImportantDates deadlineDate) ifFalse: [^ false].

^ true.
! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:21'!
deadlineDate
	^ deadlineDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:22'!
deadlineDate: conferenceDeadlineDate
	deadlineDate := conferenceDeadlineDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
endDate
	^ endDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
endDate: conferenceEndDate
	endDate := conferenceEndDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:18'!
startDate
	^ startDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
startDate: conferenceStartDate
	startDate := conferenceStartDate! !

Object subclass: #GCRating
	instanceVariableNames: 'qualis hIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCRating commentStamp: 'cms 11/21/2014 19:28' prior: 0!
A GCRating represents the qualis rating.

Instance Variables
	hIndex:		<Object>
	qualis:		<Object>

hIndex
	- xxxxx

qualis
	- xxxxx
!

!GCRating class methodsFor: 'initialize' stamp: 'pfsm 11/6/2014 23:04'!
qualis: aQualis hIndex: anHIndex
	"constructor method"

	|aRating|
	aRating:= self new.

	aRating qualis: aQualis;
				hIndex: anHIndex.

^ aRating.! !

!GCRating methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:10'!
hIndex

	^ hIndex! !

!GCRating methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:10'!
hIndex: anHIndex

	hIndex := anHIndex! !

!GCRating methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:10'!
qualis

	^ qualis! !

!GCRating methodsFor: 'accessing' stamp: 'pfsm 10/16/2014 23:11'!
qualis: aQualis

	qualis := aQualis! !

Object subclass: #GCSearchFilter
	instanceVariableNames: 'keywords categories startDate endDate deadlineDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSearchFilter commentStamp: 'cms 11/21/2014 19:28' prior: 0!
A GCSearchFilter holds a set of values that are used to filter a list of conferences.

Instance Variables
	categories:		<Object>
	deadlineDate:		<Object>
	endDate:		<Object>
	keywords:		<Object>
	startDate:		<Object>
!

!GCSearchFilter class methodsFor: 'initialize' stamp: 'pfsm 11/7/2014 17:41'!
new
	"creates a new object with all fields nil"
	| aSearchFilter |

	aSearchFilter := super new.
	aSearchFilter keywords: nil;
						categories: nil;
						startDate: nil;
						endDate: nil;
						deadlineDate: nil.

	^ aSearchFilter! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 11/6/2014 19:10'!
categories

	^ categories! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 11/6/2014 19:10'!
categories: setOfCategories

	categories := setOfCategories! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
deadlineDate

	^ deadlineDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:31'!
deadlineDate: aDeadlineDate
"set filter for searching conferences with deadline until aDeadlineDate"
	deadlineDate := aDeadlineDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
endDate

	^ endDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:32'!
endDate: anEndDate
"set filter for searching conferences with end date until anEndDate"
	endDate := anEndDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 11/6/2014 18:50'!
keywords

	^ keywords! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 11/6/2014 19:11'!
keywords: setOfKeywords

	keywords := setOfKeywords! !

!GCSearchFilter methodsFor: 'printing' stamp: 'fabcm 11/7/2014 16:46'!
printString
	"Create a string describing the contents stored on search filter"

	| contents |
	
	contents := String empty.
	contents := contents, 'Categories: '.
		[categories do:[ :aCategory | contents := contents, aCategory, ' ' ].] on: MessageNotUnderstood do: [].
		contents := contents, ' .', String cr.
	contents := contents, 'Deadline Date:: '.
		[( deadlineDate = nil ) ifFalse: [ contents := contents, (deadlineDate printFormat: #(1 2 3 $/ 1 1)) asString , ' .'].] on: MessageNotUnderstood do: [].
		contents := contents , String cr.
	contents := contents, 'Start Date: '.
		[( startDate = nil ) ifFalse: [ contents := contents, (startDate printFormat: #(1 2 3 $/ 1 1)) asString, ' .'].] on: MessageNotUnderstood do: [].
		contents := contents, String cr.
	contents := contents, 'End Date: '.
		[( endDate = nil ) ifFalse: [ contents := contents, (endDate printFormat: #(1 2 3 $/ 1 1)) asString , ' .'].] on: MessageNotUnderstood do: [].
		contents := contents, String cr.
	contents := contents, 'Search Key: '.
		[keywords do:[ :aKeyword | contents := contents, aKeyword, ' ' ].] on: MessageNotUnderstood do: [].
		contents := contents, ' .', String cr.
	
	^ contents! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
startDate

	^ startDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:35'!
startDate: aStartDate
"set filter for searching conferences with start date after  aStartDate"
	startDate := aStartDate! !

Object subclass: #GCSource
	instanceVariableNames: 'categoryFilepath crawler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSource commentStamp: 'cms 11/21/2014 19:30' prior: 0!
A GCSource represents a conference source and it contains the path to the desidered categories from it and a crawler that can actualy give us the conferences.

Instance Variables
	categoryFilepath:		<Object>
	crawler:		<Object>
!

!GCSource methodsFor: 'accessing' stamp: 'cms 11/22/2014 15:29'!
categories
	"returns all categories found in categoryFilepath. Each category must be mapped to a system category using the = character"
	|fileData categories tokens|
	[
		fileData := (StandardFileStream fileNamed: categoryFilepath ) contentsOfEntireFile.
		categories := Dictionary new.
		fileData lines do: [ :line |
			tokens := line findTokens: '='.
			categories at: (tokens at: 1) put: (tokens at: 2).	
			
		].
		^ categories.
	]
	on: Error
	do: [
		^ Dictionary new.
	]! !

!GCSource methodsFor: 'accessing' stamp: 'cms 10/14/2014 23:36'!
categoryFilepath
	 ^ categoryFilepath.! !

!GCSource methodsFor: 'accessing' stamp: 'cms 10/14/2014 23:41'!
categoryFilepath: aFilepath
	"set path to file containing categories of this source"
	categoryFilepath := aFilepath.! !

!GCSource methodsFor: 'accessing' stamp: 'cms 10/15/2014 01:01'!
crawler
	^ crawler.! !

!GCSource methodsFor: 'accessing' stamp: 'cms 10/15/2014 01:01'!
crawler: aCrawler
	crawler := aCrawler.! !

Object subclass: #GCSourceCrawler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSourceCrawler commentStamp: 'cms 11/21/2014 19:31' prior: 0!
A GCSourceCrawler is an abstract class that provides an interface and methods that all conferences' crawlers must have.

Instance Variables
!

GCSourceCrawler subclass: #GCConfSearchCrawler
	instanceVariableNames: 'baseUrl filter fetcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCConfSearchCrawler commentStamp: 'cms 11/21/2014 19:31' prior: 0!
A GCConfSearchCrawler is an crawler for getting conferences from ConfSearch website.
!

!GCConfSearchCrawler methodsFor: 'performing' stamp: 'cms 11/22/2014 16:45'!
crawl: categoryDictionary
	"returns a collection of GCConferences crawled from searchConf"
	|pageData conferences categoryEncoded|
	conferences := OrderedCollection new.
	categoryDictionary keys do: [ :category |
		categoryEncoded := category copyReplaceAll: ' ' with: '%20'.
		pageData := fetcher fetch: (baseUrl,categoryEncoded). 
		conferences addAll: (self parse: pageData from: (categoryDictionary at: category)).	].
	^ conferences.! !

!GCConfSearchCrawler methodsFor: 'accessing' stamp: 'cms 11/22/2014 14:11'!
fetcher: anFetcher
	fetcher := anFetcher.! !

!GCConfSearchCrawler methodsFor: 'initialize' stamp: 'cms 11/22/2014 16:45'!
initialize
	filter := FILTTextFilter new.
	baseUrl := 'http://confsearch.org/confsearch/faces/pages/topic.jsp?topic='.
	fetcher := WEBPageFetcher.
	^ self.! !

!GCConfSearchCrawler methodsFor: 'parsing' stamp: 'cms 11/22/2014 03:28'!
parse: page from: category
	| conferenceCollection 
	 conference importantDates locationString
	 siteCollection siteStart siteEnd 
       acronymCollection acronymStart acronymEnd 
       nameCollection nameStart nameEnd 
       dateCollection dateStart dateEnd 
	 keywordsCollection keywordsStart keywordsEnd
       locationCollection locationStart locationEnd
	categories
	|

	categories := OrderedCollection new.
	categories add: category asLowercase.	
		
	siteStart := 'wwwColumn">'.
	siteEnd := '</'.
	
	keywordsStart := 'Keywords: '.
	keywordsEnd := '</span&gt;'.

	acronymStart := 'return false;" class="venue'.
	acronymEnd := '</a'.

	nameStart := 'venueKeywords">'.
	nameEnd := '<'.


	locationStart := 'td class="locationColumn">'.
	locationEnd := '</td>'.

	dateStart := 'td class="dateColumn">'.
	dateEnd := '</td>'.

	siteCollection := filter filterText: page  between: siteStart  and: siteEnd.

	acronymCollection := filter filterText: page  between: acronymStart and: acronymEnd.

	nameCollection := filter filterText: page  between: nameStart and: nameEnd.

	keywordsCollection := filter filterText: page  between: keywordsStart and: keywordsEnd.

	locationCollection := filter filterText: page  between: locationStart and: locationEnd.

	dateCollection := filter filterText: page  between: dateStart and: dateEnd.

	conferenceCollection := OrderedCollection new.
.
	1 to: acronymCollection size  do: [ :i |
		conference := GCConference new.
		conference acronym: (self stripTagsFrom:(acronymCollection at: i)).
		conference name:  (nameCollection at: i).
		conference url: (self parseUrl: (siteCollection at: i)).
		
		locationString := self stripTagsFrom: (locationCollection at: i).
		conference venue: ( self parseVenue: locationString).
		importantDates := GCImportantDates new.
		conference categories: categories.
		importantDates deadlineDate: (self parseDate: (self stripTagsFrom: (dateCollection at: 4*(i-1) + 1 ))).
		importantDates startDate: (self parseDate: (self stripTagsFrom: (dateCollection at: 4*(i-1) +  3))).
		importantDates endDate: (self parseDate: (self stripTagsFrom: (dateCollection at: (4*(i-1)  + 4)))).
		importantDates startDate ~= nil ifTrue: [
			conference edition: (importantDates startDate year).
		].
		conference importantDates: importantDates.
		conference keywords: ((keywordsCollection at: i) findTokens: ', ').
		conferenceCollection add: conference.
	].
	^ conferenceCollection.

! !

!GCConfSearchCrawler methodsFor: 'parsing' stamp: 'cms 11/22/2014 03:18'!
parseDate: anString
	"returns a date from a string if the given string is valid"
	|date|
	[
		(anString ~= nil  & anString isEmpty not ) ifTrue: [
		date := (Date readFrom: (anString) readStream).
		]
	]
	on: Error do:
	[].
	^ date.! !

!GCConfSearchCrawler methodsFor: 'parsing' stamp: 'cms 11/8/2014 15:12'!
parseUrl: anString
	|urlStart urlEnd urls|
	urlStart := 'href="'.
	urlEnd := '" target'.
	urls :=  filter filterText: anString  between: urlStart  and: urlEnd.
	(urls = nil or: urls size = 0) ifTrue:[
		^ ''.	
	].
	^ urls at: 1.! !

!GCConfSearchCrawler methodsFor: 'parsing' stamp: 'cms 11/21/2014 19:42'!
parseVenue: locationString
	"returns a GCVenue with the information contained in locationString"
	|locationTokens venue|
	locationTokens := locationString findTokens: ','.
	venue := GCVenue new.
	locationTokens size > 0 ifTrue: [
		venue country: (locationTokens at: 1) withBlanksTrimmed.
		locationTokens size > 1 ifTrue: [
			venue city: (locationTokens at: 2) withBlanksTrimmed .
		].
	].
	^ venue.! !

!GCConfSearchCrawler methodsFor: 'parsing' stamp: 'cms 11/21/2014 19:42'!
stripTagsFrom: anString
	"remove all html tags in anString"
	^ (anString copyWithRegex: '</.+'  matchesReplacedWith: '' ) copyWithRegex: '.+>' matchesReplacedWith: ''.! !

GCSourceCrawler subclass: #GCQualisCrawler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCQualisCrawler commentStamp: 'cms 11/21/2014 19:32' prior: 0!
A GCQualisCrawler is an crawler responsible for getting conferences from a file that contains a list of conferences and its ratings.

Instance Variables
!

!GCQualisCrawler methodsFor: 'performing' stamp: 'pfsm 11/26/2014 00:26'!
crawl: categories
 "crawl conference from qualis. Qualis is very much a static source (the hindex and rating do not change very often) so instead of crawling from a web   source we use a local file. For now categories are not used "
 | qualis conferencesCollection sheet conference rating|		
		qualis := SSSpreadsheetData fromFile: 'GODCallTests_data/Conferncias_completo.csv'.	
		conferencesCollection := OrderedCollection new.
		sheet := qualis getSheet: 1.	
		sheet deleteRow: 1. "header"	
		sheet do: [:row | 		
			conference := GCConference new.		
			conference acronym: (row getCell: 'A') value.		
			conference name: (row getCell: 'B') value.		
			rating := GCRating new.		
			rating hIndex: (row getCell: 'C') value.		
		     rating qualis: (row getCell: 'D') value.		
			conference rating: rating.
			conference origin: 'qualis'.
			conferencesCollection add: conference.   	
		].	
		^  conferencesCollection.! !

!GCSourceCrawler methodsFor: 'performing' stamp: 'cms 11/21/2014 19:41'!
crawl: categories
	"returns a collection of GCConferences crawled from this source"
	^ self subclassResponsibility.! !

GCSourceCrawler subclass: #GCWikiCfpCrawler
	instanceVariableNames: 'filter fetcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCWikiCfpCrawler commentStamp: 'cms 11/21/2014 19:32' prior: 0!
A GCWikiCfpCrawler is an crawler responsible for getting conferences from WikiCfp website.

Instance Variables
!

!GCWikiCfpCrawler methodsFor: 'generating urls' stamp: 'pfsm 11/22/2014 19:17'!
conferencesUrlsFrom: categoryUrl
	"explore each page of categoryUrl and extract the conference links in these pages"

	|filteredStrings emptyPage pageNumber conferencesUrls categoryStartMark categoryEndMark pageData wikiCfpUrlPrefix|

	filteredStrings := String empty.
	emptyPage := false.
	pageNumber:=1.
	"conferencesUrls := Set new."
	conferencesUrls := OrderedCollection new.
	wikiCfpUrlPrefix :='http://www.wikicfp.com'.
	categoryStartMark := '<td rowspan="2" align="left"><a href="'.
	categoryEndMark := '">'.

	"this loop fills in conferenceUrls with the links to each conference shown in the webpage pointed by categoryUrl "
	[emptyPage = false]
		whileTrue: [ 	"Web interface expects input url as String"
							pageData := fetcher fetch: ((categoryUrl asString),'&page=', (pageNumber asString)). 
							filteredStrings := filter filterText: (pageData) between: categoryStartMark and: categoryEndMark.
							
							(filteredStrings size > 0)
								ifTrue: [	
												filteredStrings do: [:conferenceUrlSuffix | 	(conferenceUrlSuffix beginsWith:'/')
																													ifTrue:[conferencesUrls add: (wikiCfpUrlPrefix,conferenceUrlSuffix) asUrl]
																													ifFalse:[conferencesUrls add: (conferenceUrlSuffix) asUrl].
																												].
												pageNumber := pageNumber+1.
											]
								ifFalse:[emptyPage := true ].
						].
	

	^ conferencesUrls.! !

!GCWikiCfpCrawler methodsFor: 'performing' stamp: 'pfsm 11/24/2014 19:09'!
crawl: categories
	"return a collection of GCConferences crawled from WikiCFP"
	
	| page categoriesUrls conferencesUrls conferencesList|


	categoriesUrls := self urlsFrom: categories.
	
	conferencesUrls := OrderedCollection new.
	categoriesUrls do: [ :categoryUrl |	conferencesUrls := conferencesUrls union: (self conferencesUrlsFrom: categoryUrl) ].

	conferencesList := OrderedCollection new.
	conferencesUrls do:[ :confUrl | 	"Web interface expects input url as String"
													page  := fetcher fetch: (confUrl asString). 
													conferencesList addIfNotPresent: (self parse: page).
								].
^ conferencesList.! !

!GCWikiCfpCrawler methodsFor: 'accessing' stamp: 'pfsm 11/22/2014 21:09'!
fetcher: aFetcher
	fetcher:=aFetcher.! !

!GCWikiCfpCrawler methodsFor: 'initialize' stamp: 'pfsm 11/22/2014 18:57'!
initialize
	filter := FILTTextFilter new.
	fetcher := WEBPageFetcher.
	^ self.! !

!GCWikiCfpCrawler methodsFor: 'parsing' stamp: 'pfsm 11/26/2014 12:39'!
parse: page
	"given a page (html code) of a conference in WikiCFP, this method returns a GCConference object whose its attributes are extracted from  page "

	| filteredStrings acronymAndEditionSMark acronymAndEditionEMark nameSMark nameEMark urlSMark urlEMark startAndDeadlineSMark startAndDeadlineEMark endSMark endEMark cityAndCountrySMark cityAndCountryEMark categoriesSMark categoriesEMark cfpSMark cfpEMark conference|


	acronymAndEditionSMark :='"dc:title" content="'.
	acronymAndEditionEMark :='">'.
	nameSMark :='"dc:description" content="'.
	nameEMark :='">'.
	urlSMark :='"dc:source" content="'.
	urlEMark :='">'.
	startAndDeadlineSMark :='"v:startDate" content="'.
	startAndDeadlineEMark :='T'.
	endSMark :='"v:endDate" content="'.
	endEMark :='T'.
	cityAndCountrySMark :='"v:locality" content="'.
	cityAndCountryEMark :='">'.
	categoriesSMark:='call?conference='.
	categoriesEMark:='">'.	
	cfpSMark:='<div class="cfp" align="left">'.
	cfpEMark:='</div>'.

	filteredStrings := String empty.
	
	conference := GCConference new.

	filteredStrings := filter filterText: page  between: acronymAndEditionSMark  and: acronymAndEditionEMark.
	filteredStrings := (filteredStrings at:1) findTokens: String space.
	conference acronym: (filteredStrings at:1).
	conference edition: (filteredStrings at:2).

	filteredStrings := filter filterText: page  between: nameSMark and: nameEMark.
	conference name: (filteredStrings at:1).
	
	filteredStrings := filter filterText: page  between: urlSMark and: urlEMark.
	conference url: ((filteredStrings at:1) asUrl).

	conference importantDates: (self parseImportantDates:( filter filterText: page  between: startAndDeadlineSMark and: startAndDeadlineEMark) and:(filter filterText: page  between: endSMark and: endEMark)).
	conference venue: (self parseVenue: (filter filterText: page  between: cityAndCountrySMark and: cityAndCountryEMark)).
	conference categories: (self parseCategories: (filter filterText: page  between: categoriesSMark and: categoriesEMark)).
	conference content: (self parseContent: (filter filterText: page  between: cfpSMark and: cfpEMark)).
	conference origin:'wikicfp'.

^ conference.! !

!GCWikiCfpCrawler methodsFor: 'parsing' stamp: 'pfsm 11/7/2014 16:59'!
parseCategories: categoriesStrings

	| categoriesSet |
	categoriesSet := Set new.

	categoriesStrings do:[:category | categoriesSet add: ((category copyReplaceAll: '%20' with: String space) asLowercase)].

^categoriesSet! !

!GCWikiCfpCrawler methodsFor: 'parsing' stamp: 'pfsm 11/7/2014 17:16'!
parseContent: contentStrings

^(contentStrings size >0) ifTrue:[contentStrings at:1]! !

!GCWikiCfpCrawler methodsFor: 'parsing' stamp: 'pfsm 11/7/2014 16:33'!
parseImportantDates: startAndDeadlineStrings and: endStrings

	| importantDates |
	importantDates := GCImportantDates new.

	importantDates startDate: ((startAndDeadlineStrings size > 0) ifTrue:[ (startAndDeadlineStrings at:1) asDate]).
	importantDates deadlineDate: ((startAndDeadlineStrings size > 1) ifTrue:[ (startAndDeadlineStrings at:2) asDate]).
	importantDates endDate: ((endStrings size > 0) ifTrue:[(endStrings at:1) asDate]).

^ importantDates! !

!GCWikiCfpCrawler methodsFor: 'parsing' stamp: 'pfsm 11/7/2014 16:55'!
parseVenue: locationString
	"create a a venue object by parsing a location string"

	| venue venueString venueTokens |
	venue := GCVenue new.
	venueTokens := OrderedCollection new.

	(locationString  size > 0)
		ifTrue: [	venueString := (locationString at:1) copyReplaceAll: '(' with:','.
					venueString := (venueString) copyReplaceAll: ')' with: String empty.
					venueTokens := (venueString) findTokens: ','].
	
	venue city: ((venueTokens size > 0) 
							ifTrue:[(venueTokens at: 1) withBlanksTrimmed]
							ifFalse:[nil]).

	"sometimes there is only the city of the event"
	venue country: ((venueTokens size > 1) 
							ifTrue: [(venueTokens at: (venueTokens size)) withBlanksTrimmed] 
							ifFalse:[nil]).

^ venue! !

!GCWikiCfpCrawler methodsFor: 'generating urls' stamp: 'pfsm 11/22/2014 20:38'!
urlsFrom: categories
	"create a WikiCFP url for each category in the list of categories"
	|urls wikiCfpUrlPrefix|

	wikiCfpUrlPrefix := 'http://www.wikicfp.com/cfp/call?conference='. 
	urls := OrderedCollection new.

	categories keys do: [ :category |	category := category copyReplaceAll: String space with: '%20'.
											urls add: ((wikiCfpUrlPrefix,category) asUrl).
						].

	^ urls.

! !

Object subclass: #GCSourceCreator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSourceCreator commentStamp: 'cms 11/21/2014 19:33' prior: 0!
A GCSourceCreator creates all sources used in GODCall.

Instance Variables
!

!GCSourceCreator class methodsFor: 'create sources' stamp: 'pfsm 11/26/2014 12:10'!
createAll
	"returns a list of sources ordered by priority"
	|all|
	all := OrderedCollection new.	
	all add: (self createWikiCfp).
	all add: (self createConfSearch).
	all add: (self createQualis).

	^ all.! !

!GCSourceCreator class methodsFor: 'create sources' stamp: 'cms 11/9/2014 20:01'!
createConfSearch
	"creates and returns a instance of GCSource that represents the source WikiCfp"
	|source|
	source := GCSource new.
	source categoryFilepath: 'GODCallTests_data/confSearch_categories.txt'.
	source crawler: GCConfSearchCrawler new initialize.
	^ source.
! !

!GCSourceCreator class methodsFor: 'create sources' stamp: 'cms 11/8/2014 22:39'!
createQualis
	"creates and returns a instance of GCSource that represents the source Qualis"
	|source|
	source := GCSource new.
	source categoryFilepath: ''.
	source crawler: GCQualisCrawler new.
	^ source.! !

!GCSourceCreator class methodsFor: 'create sources' stamp: 'pfsm 11/22/2014 18:58'!
createWikiCfp
	"creates and returns a instance of GCSource that represents the source WikiCfp"
	|source|
	source := GCSource new.
	source categoryFilepath: 'GODCallTests_data/wikiCfp_categories.txt'.
	source crawler: GCWikiCfpCrawler new initialize.
	^ source.! !

Object subclass: #GCVenue
	instanceVariableNames: 'place city country'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCVenue commentStamp: 'cms 11/21/2014 19:33' prior: 0!
A GCVenue represents the venue of a conference!

!GCVenue methodsFor: 'comparing' stamp: 'pfsm 10/16/2014 16:25'!
= aVenue
	"compare two venues"
	
	self == aVenue ifTrue: [^ true].	"stop recursion"
	(self place = aVenue place) ifFalse: [^ false].
	(self city = aVenue city) ifFalse: [^ false].
	(self country = aVenue country) ifFalse: [^ false].

^ true.

! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
city
	^ city! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
city: conferenceCity
	city := conferenceCity! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
country
	^ country! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
country: conferenceCountry
	country := conferenceCountry! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
place
	^ place! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:26'!
place: conferencePlace
	place := conferencePlace! !

Object subclass: #GODCall
	instanceVariableNames: 'component categories searchForm categoryForms deadlineDate startDate endDate filter conferenceCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GODCall commentStamp: 'cms 11/21/2014 19:34' prior: 0!
A GODCall is the class that can be used by external sources. It provides methods for user interaction.

Instance Variables
	categories:		<Object>
!

!GODCall class methodsFor: 'extracting' stamp: 'pfsm 11/6/2014 19:14'!
extractKeywordsFrom: aString
	"extract the tokens of aString and put them in normalized format"
	|normalizedTokens tokens|

	tokens := aString findTokens: String space.
	normalizedTokens := Set new.	

	tokens do:[ :token | normalizedTokens add: ((token withBlanksTrimmed) asLowercase)].

	 ^normalizedTokens.
! !

!GODCall methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 01:25'!
categories

	^ categories! !

!GODCall methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 01:25'!
categories: anObject

	categories := anObject! !

!GODCall methodsFor: 'initialize' stamp: 'fabcm 11/28/2014 18:01'!
component: aComponent

	component := aComponent.! !

!GODCall methodsFor: 'trash' stamp: 'fabcm 11/28/2014 15:36'!
crawlConferences

	! !

!GODCall methodsFor: 'trash' stamp: 'fabcm 11/28/2014 14:54'!
initialize
! !

!GODCall methodsFor: 'initialize' stamp: 'fabcm 11/29/2014 18:34'!
initialize: aComponent
	"crawlConferences for the first time if the database is empty. aComponent is an object necessary to comunicate with seaside. "

      component:= aComponent.
	self isConferencesCrawled ifFalse: [GCCrawler run.].
	self categories: GCCrawler categories.! !

!GODCall methodsFor: 'initialize' stamp: 'fabcm 11/28/2014 15:05'!
isConferencesCrawled
	"Check if the conferences are stored in the database"

	| loadedCollection |

	GDBClientController initializeSession.
	GDBClientController refresh.
	loadedCollection := GDBClientController loadConferences.
	GDBClientController release.
	
	^ loadedCollection ~= nil.! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:37'!
makeCategoryForms
"This method makes one checkboxItem for each category, to be show in the first page formulary"


	|  checkboxItem |

	categoryForms := WEBCheckBox new.
	categoryForms label: 'Categories:'.
	categories do: [ :category |
		checkboxItem := WEBCheckBoxItem new.
		checkboxItem label: category.
		checkboxItem value: false.
		categoryForms add: checkboxItem.
	].! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:43'!
makeSearchFilter
"This method makes the search filter with the parameters of the first page's formulary."

	| filter categoriesSet | 

	filter := GCSearchFilter new.
	filter keywords: (GODCall extractKeywordsFrom: searchForm value).
	filter keywords isEmpty ifTrue:[
		filter keywords: nil.
	].
	
	categoriesSet := Set new.
	categoryForms items do:[ :item |
		item value ifTrue:[ 
			categoriesSet add: item label.].
	].
	categoriesSet isEmpty ifFalse:[
		filter categories: categoriesSet.
	].
	
	filter deadlineDate: deadlineDate value.
	filter startDate: startDate value.
	filter endDate: endDate value.
	
	^ filter.! !

!GODCall methodsFor: 'trash' stamp: 'fabcm 11/29/2014 09:40'!
renderContent
! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:37'!
renderFirstPage
"This message constructs the WEBPage with the initial formulary"

	| page iParagraph iForm submit|
	
	iParagraph := WEBParagraph new.
	iParagraph value: 'Search for a conference in our database!! Choose your filter preferences:'.
	
	searchForm := WEBInputText new.
	searchForm label: 'Search Key:'.
	searchForm value: nil.
	
	self makeCategoryForms.
	
	deadlineDate := WEBDate new.
	deadlineDate label: 'Deadline Date:'.
	deadlineDate value: (Date year:2100 month: 1 day: 1).
	
	startDate := WEBDate new.
	startDate label: 'Start Date:'.
	startDate value: (Date year:1900 month: 1 day: 1).
	
	endDate := WEBDate new.
	endDate label: 'End Date:'.
	endDate value: (Date year:2100 month: 1 day: 1).
	
	submit := WEBSubmitButton new.
	submit label: 'Search'.
	submit callback: self.
	
	iForm := WEBForm new.
	iForm initialize.
	iForm add: iParagraph.
	iForm add: searchForm.
	iForm add: categoryForms.
	iForm add: deadlineDate.
	iForm add: startDate.
	iForm add: endDate.
	iForm add: submit.
	
	page := WEBPage new.
	page initialize.
	page title: 'Welcome to Gods Call'.
	page add: iForm.
	
	^page! !

!GODCall methodsFor: 'trash' stamp: 'fabcm 11/29/2014 09:39'!
renderOutputPage: conferencesSpreadsheet
! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:38'!
renderSecondPage
"This message constructs the WEBPage with the results table"

	| page iForm iParagraph iSpreadsheet buttonList spread ibutton response linkList iAnchor|

	iParagraph := WEBParagraph new.
	iParagraph value: 'Here are the conferences that match your query.'.
	
	buttonList:=OrderedCollection new.
	linkList:=OrderedCollection new.
	conferenceCollection do:[ :conference |
		response := GCConferenceResponse new.
		response initialize: component withConference: conference.
		ibutton := WEBSubmitButton new.
		ibutton label: 'Details'.
		ibutton callback: response.
		buttonList add: ibutton.
		iAnchor := WEBAnchor new.
		iAnchor label: conference url.
		iAnchor value: conference url.
		linkList add: iAnchor.
	].

	spread := GCConference asSpreadsheet: conferenceCollection.	
	iSpreadsheet := WEBSpreadSheet new initialize: spread withImages: buttonList withLinks: linkList.
	iSpreadsheet sheetNumber: 1.
	iSpreadsheet header: true.
	iSpreadsheet width: 1000.
	iSpreadsheet cellBorder: 1.
	iSpreadsheet rowBorder: 0.
	iSpreadsheet imageHeader: 'Details'.
	iSpreadsheet linkHeader: 'Site'.
	
	iForm := WEBForm new.
	iForm initialize.
	iForm add: iParagraph.
	iForm add: iSpreadsheet.
	
	page:= WEBPage new.
	page initialize.
	page title: 'Response'.
	page add: iForm.
	
	^page.! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/29/2014 18:44'!
save
"This message is triggered when the uses clicks in the button on the end of first page and is responsible to call the filter function and call the second page."

	filter := self makeSearchFilter.
	
	conferenceCollection := self submitForm: filter.
	component call: (GCSecondPage new object: self).
	
! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 11/28/2014 19:14'!
submitForm: searchFilter
	"executes a user's query defined in the searchFilter and returns a spreadsheet"

	|godConferenceCollection filter|

	GDBClientController initializeSession.
	GDBClientController refresh.
	godConferenceCollection := GDBClientController loadConferences.
	GDBClientController release.

	filter := FILTConferenceFilter new.
	godConferenceCollection := filter filter: godConferenceCollection byGCSearchFilter: searchFilter.

     ^ godConferenceCollection.! !
GCFirstPage initialize!
